package org.subshare.gui.resolvecollision.collision;

import static co.codewizards.cloudstore.core.util.DateUtil.*;
import static java.util.Objects.*;
import static org.subshare.gui.util.FxmlUtil.*;

import java.util.Date;

import org.subshare.core.dto.CollisionPrivateDto;
import org.subshare.gui.resolvecollision.CollisionDtoWithPlainHistoCryptoRepoFileDto;
import org.subshare.gui.wizard.WizardPageContentGridPane;

import javafx.beans.InvalidationListener;
import javafx.fxml.FXML;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextArea;

public class CollisionPane extends WizardPageContentGridPane {

	private final CollisionData collisionData;

	@FXML
	private CheckBox resolvedCheckBox;

	@FXML
	private TextArea commentTextArea;

	public CollisionPane(final CollisionData collisionData) {
		loadDynamicComponentFxml(CollisionPane.class, this);
		this.collisionData = requireNonNull(collisionData, "collisionData");

		final CollisionDtoWithPlainHistoCryptoRepoFileDto collisionDtoWithPlainHistoCryptoRepoFileDto = collisionData.getCollisionDtoWithPlainHistoCryptoRepoFileDto();
		final CollisionPrivateDto collisionPrivateDto = collisionDtoWithPlainHistoCryptoRepoFileDto.getCollisionPrivateDto();

		resolvedCheckBox.setSelected(collisionPrivateDto.getResolved() != null);
		resolvedCheckBox.selectedProperty().addListener((InvalidationListener) observable -> updateResolved());

		commentTextArea.setText(collisionPrivateDto.getComment());
		commentTextArea.textProperty().addListener((InvalidationListener) observable -> updateComment());
	}

	private void updateResolved() {
		final CollisionDtoWithPlainHistoCryptoRepoFileDto collisionDtoWithPlainHistoCryptoRepoFileDto = collisionData.getCollisionDtoWithPlainHistoCryptoRepoFileDto();
		final CollisionPrivateDto collisionPrivateDto = collisionDtoWithPlainHistoCryptoRepoFileDto.getCollisionPrivateDto();

		final boolean newResolved = resolvedCheckBox.selectedProperty().get();
		if (newResolved)
			collisionPrivateDto.setResolved(now());
		else
			collisionPrivateDto.setResolved(null);
	}

	private void updateComment() {
		final CollisionDtoWithPlainHistoCryptoRepoFileDto collisionDtoWithPlainHistoCryptoRepoFileDto = collisionData.getCollisionDtoWithPlainHistoCryptoRepoFileDto();
		final CollisionPrivateDto collisionPrivateDto = collisionDtoWithPlainHistoCryptoRepoFileDto.getCollisionPrivateDto();

		final String newComment = commentTextArea.getText();
		collisionPrivateDto.setComment(newComment);
	}

	@Override
	public void requestFocus() {
		super.requestFocus();
		resolvedCheckBox.requestFocus();
	}

	@Override
	protected boolean isComplete() {
		return true;
	}
}
